// -*- Mode: C; tab-width: 8 -*-

// $Id: sendfilemodule.c,v 3.1 2000/02/01 00:51:51 rushing Exp $

#include "Python.h"
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/uio.h>

//
// sendfile(), meant for use with a non-blocking socket
//

#if defined (__linux__)

static PyObject *
py_sendfile (PyObject * self, PyObject * args)
{
  int fd;
  int sock;
  long o,n;
  if (!PyArg_ParseTuple (args, "iill", &fd, &sock, &o, &n)) {
    return NULL;
  } else {
    off_t offset = o;
    size_t nbytes = n;
    ssize_t result;

    result = sendfile (sock, fd, &offset, nbytes);

    if (result < 0) {
      if (errno == EAGAIN) {
	return PyInt_FromLong (0);
      } else {
	PyErr_SetFromErrno (PyExc_OSError);
      }
    } else {
      return PyInt_FromLong (result);
    }
  }
}

#elif defined (__FreeBSD__)
// XXX support the header/trailer stuff.
static PyObject *
py_sendfile (PyObject * self, PyObject * args)
{
  int fd;
  int sock;
  long o,n;
  if (!PyArg_ParseTuple (args, "iill", &fd, &sock, &o, &n)) {
    return NULL;
  } else {
    off_t offset = o;
    size_t nbytes = n;
    off_t sbytes;
    int result;

    result = sendfile (fd, sock, offset, nbytes, NULL, &sbytes, 0);
    if (result == -1) {
      if (errno == EAGAIN) {
	return PyInt_FromLong (sbytes);
      } else {
	PyErr_SetFromErrno (PyExc_OSError);
      }
    } else {
      return PyInt_FromLong (sbytes);
    }
  }
}

#else
#error ("unknown platform")
#endif


// <off_t> and <size_t> are 64 bits on FreeBSD.
// If you need this feature, then look at PyLong_FromUnsignedLongLong.
// You'll have to specifically check for the PyLong type, etc...
// Also, LONG_LONG is a configure variable (i.e., the conversion function
// might not be available).

static PyMethodDef sendfile_methods[] = {
  {"sendfile",		py_sendfile,		1},
  {NULL,		NULL}		/* sentinel */
};

DL_EXPORT(void)
initsendfile()
{
  PyObject *m, *d;
  m = Py_InitModule("sendfile", sendfile_methods);
}
